@@ -19,6 +19,10 @@
 #ifndef GRPC_CORE_TSI_SSL_TYPES_H
 #define GRPC_CORE_TSI_SSL_TYPES_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* A collection of macros to cast between various integer types that are
  * used differently between BoringSSL and OpenSSL:
  * TSI_INT_AS_SIZE(x):  convert 'int x' to a length parameter for an OpenSSL
@@ -37,4 +41,8 @@
 #define TSI_SIZE_AS_SIZE(x) ((int)(x))
 #endif
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* GRPC_CORE_TSI_SSL_TYPES_H */
